@@ -1,15 +1,18 @@
 from abc import ABC, abstractmethod
+from typing_extensions import (
+    NotRequired,
+    Literal,
+    Protocol,
+    runtime_checkable,
+    TypedDict,
+)
 from typing import (
     List,
-    Literal,
     Optional,
     Union,
     Any,
-    Protocol,
     overload,
-    runtime_checkable,
     AsyncGenerator,
-    TypedDict,
     Tuple,
     Dict,
     Callable,
@@ -22,6 +25,7 @@ from rx.core.typing import Observable
 from PIL import Image
 import numpy as np
 
+
 Callback = Union[
     Callable[..., Any],
     Callable[..., Coroutine[Any, Any, Any]],
@@ -29,52 +33,54 @@ Callback = Union[
 
 
 # schema
-class JsonBaseSchema(TypedDict):
-    type: Any
-    key: Optional[str]
-    title: Optional[str]
-    description: Optional[str]
-    required: Optional[bool]
-    readonly: Optional[bool]
-    placeholder: Optional[str]
-    hidden: Optional[bool]
-    group: Optional[str]
-    defaultValue: Optional[Union[str, int, bool]]
+JsonSchemaType = Literal["string", "number", "boolean", "object", "array"]
 
 
-class PluginJsonBaseSchema(JsonBaseSchema):
-    store: Optional[bool]
-    onGet: Any
-    onSet: Any
+class JsonBaseSchema(TypedDict, total=False):
+    key: str
+    title: str
+    description: str
+    required: bool
+    readonly: bool
+    placeholder: str
+    hidden: bool
+    group: str
+    defaultValue: Union[str, int, bool]
+
+
+class PluginJsonBaseSchema(JsonBaseSchema, total=False):
+    store: bool
+    onSet: NotRequired[Callable[[Any, Any], Awaitable[Union[None, Any]]]]
+    onGet: NotRequired[Callable[[], Awaitable[Union[Any, None]]]]
 
 
 class PluginJsonSchemaString(PluginJsonBaseSchema):
-    type: str
-    format: Optional[str]
-    minLength: Optional[int]
-    maxLength: Optional[int]
+    type: Literal["string"]
+    format: NotRequired[str]
+    minLength: NotRequired[int]
+    maxLength: NotRequired[int]
 
 
 class PluginJsonSchemaNumber(PluginJsonBaseSchema):
-    type: int
-    minimum: Optional[int]
-    maximum: Optional[int]
+    type: Literal["number"]
+    minimum: NotRequired[int]
+    maximum: NotRequired[int]
 
 
 class PluginJsonSchemaBoolean(PluginJsonBaseSchema):
-    type: bool
+    type: Literal["boolean"]
 
 
 class PluginJsonSchemaEnum(PluginJsonBaseSchema):
-    type: str
+    type: Literal["string"]
     enum: List[str]
-    multiple: Optional[bool]
+    multiple: NotRequired[bool]
 
 
 class PluginJsonSchemaObject(PluginJsonBaseSchema):
-    type: Dict[str, Any]
-    opened: Optional[bool]
-    properties: Optional[Dict[str, "PluginJsonSchema"]]
+    type: Literal["object"]
+    opened: NotRequired[bool]
+    properties: NotRequired["PluginJsonSchemaForm"]
 
 
 PluginJsonSchema = Union[
@@ -94,32 +100,32 @@ class PluginRootSchema(TypedDict):
 
 
 class JsonSchemaString(JsonBaseSchema):
-    type: str
-    format: Optional[str]
-    minLength: Optional[int]
-    maxLength: Optional[int]
+    type: Literal["string"]
+    format: NotRequired[str]
+    minLength: NotRequired[int]
+    maxLength: NotRequired[int]
 
 
 class JsonSchemaNumber(JsonBaseSchema):
-    type: int
-    minimum: Optional[int]
-    maximum: Optional[int]
+    type: Literal["number"]
+    minimum: NotRequired[int]
+    maximum: NotRequired[int]
 
 
 class JsonSchemaBoolean(JsonBaseSchema):
-    type: bool
+    type: Literal["boolean"]
 
 
 class JsonSchemaEnum(JsonBaseSchema):
-    type: str
+    type: Literal["string"]
     enum: List[str]
-    multiple: Optional[bool]
+    multiple: NotRequired[bool]
 
 
 class JsonSchemaObject(JsonBaseSchema):
-    type: Dict[str, Any]
-    opened: Optional[bool]
-    properties: Optional[Dict[str, "JsonSchema"]]
+    type: Literal["object"]
+    opened: NotRequired[bool]
+    properties: NotRequired["JsonSchemaForm"]
 
 
 class JsonSchemaButton(TypedDict):
@@ -128,7 +134,7 @@ class JsonSchemaButton(TypedDict):
 
 
 class JsonSchemaObjectWithButtons(JsonSchemaObject):
-    buttons: Optional[Tuple[JsonSchemaButton, Optional[JsonSchemaButton]]]
+    buttons: NotRequired[Tuple[JsonSchemaButton, NotRequired[JsonSchemaButton]]]
 
 
 class JsonSchemaAnyOf(TypedDict):
@@ -136,9 +142,9 @@ class JsonSchemaAnyOf(TypedDict):
 
 
 class JsonSchemaArray(JsonBaseSchema):
-    type: List[Any]
-    opened: Optional[bool]
-    items: Optional[Union["JsonSchema", JsonSchemaAnyOf]]
+    type: Literal["array"]
+    opened: NotRequired[bool]
+    items: NotRequired[Union["JsonSchema", JsonSchemaAnyOf]]
 
 
 JsonSchema = Union[
@@ -152,8 +158,7 @@ JsonSchema = Union[
 ]
 
 
-class JsonSchemaForm(TypedDict):
-    pass
+JsonSchemaForm = Dict[str, JsonSchema]
 
 
 class RootSchema(TypedDict):
@@ -252,7 +257,7 @@ CameraType = Literal["camera", "doorbell"]
 
 class BaseCamera(TypedDict):
     _id: str
-    nativeId: Optional[str]
+    nativeId: NotRequired[str]
     pluginId: str
     name: str
     disabled: bool
@@ -284,8 +289,8 @@ class BaseState(TypedDict):
 
 
 class Detection(TypedDict):
-    id: Optional[str]
-    label: Optional[str]
+    id: NotRequired[str]
+    label: NotRequired[str]
     boundingBox: Tuple[int, int, int, int]
 
 
@@ -449,6 +454,12 @@ class CameraPrebufferDelegate(ABC):
 
 
 class CameraSource(Protocol):
+    # CameraInput
+    _id: str
+    name: str
+    roles: List["CameraRoles"]
+    urls: StreamUrls
+
     async def is_prebuffering(self, type: str) -> bool: ...
     async def get_prebuffer_raw_url(self, type: str) -> Optional[str]: ...
     async def get_stream_info(self) -> Optional[Dict[str, Any]]: ...
@@ -462,8 +473,8 @@ class ImageCrop(TypedDict):
 
 
 class ImageResize(TypedDict, total=False):
-    width: Optional[int]
-    height: Optional[int]
+    width: NotRequired[int]
+    height: NotRequired[int]
     filter: str
 
 
@@ -476,8 +487,8 @@ class ImageOptions(TypedDict, total=False):
 
 class IceServer(TypedDict):
     urls: List[str]
-    username: Optional[str]
-    credential: Optional[str]
+    username: NotRequired[str]
+    credential: NotRequired[str]
 
 
 class FrameMetadata(TypedDict):
@@ -509,7 +520,7 @@ class VideoFrame(Protocol):
 class CameraDevice(Protocol):
     id: str
     plugin_id: str
-    native_id: Optional[str]
+    native_id: Optional[str] = None
     state: bool
     disabled: bool
     name: str
@@ -543,8 +554,8 @@ class CameraDevice(Protocol):
     snapshot_source: CameraSource
     recording_source: CameraSource
     detection_source: CameraSource
-    delegate: Optional[CameraDelegate]
-    prebuffer_delegate: Optional[CameraPrebufferDelegate]
+    delegate: Optional[CameraDelegate] = None
+    prebuffer_delegate: Optional[CameraPrebufferDelegate] = None
 
     async def connect(self) -> None: ...
     async def disconnect(self) -> None: ...
@@ -627,14 +638,14 @@ class CameraDevice(Protocol):
 # camera create config
 class BaseCameraConfig(TypedDict):
     name: str
-    nativeId: Optional[str]
-    isCloud: Optional[bool]
-    hasLight: Optional[bool]
-    hasSiren: Optional[bool]
-    hasBinarySensor: Optional[bool]
-    hasBattery: Optional[bool]
-    disabled: Optional[bool]
-    info: Optional[CameraInformation]
+    nativeId: NotRequired[str]
+    isCloud: NotRequired[bool]
+    hasLight: NotRequired[bool]
+    hasSiren: NotRequired[bool]
+    hasBinarySensor: NotRequired[bool]
+    hasBattery: NotRequired[bool]
+    disabled: NotRequired[bool]
+    info: NotRequired[CameraInformation]
 
 
 class CameraInputSettings(TypedDict):
@@ -645,11 +656,11 @@ class CameraInputSettings(TypedDict):
 
 
 class CameraConfigWithSources(BaseCameraConfig):
-    sources: Optional[List[CameraInputSettings]]
+    sources: NotRequired[List[CameraInputSettings]]
 
 
 class CameraConfigWithDelegate(BaseCameraConfig):
-    delegate: Optional[CameraDelegate]
+    delegate: NotRequired[CameraDelegate]
 
 
 CameraConfig = Union[CameraConfigWithSources, CameraConfigWithDelegate]
@@ -774,8 +785,8 @@ class CameraStorage(Protocol):
     ) -> Union[Awaitable[P], P, None]: ...
     async def set_value(self, path: str, new_value: Any) -> None: ...
     def has_value(self, path: str) -> bool: ...
-    async def get_config(self) -> SchemaConfig: ...
-    async def set_config(self, new_config: Dict[str, Any]) -> None: ...
+    async def getConfig(self) -> SchemaConfig: ...
+    async def setConfig(self, new_config: Dict[str, Any]) -> None: ...
     @overload
     async def add_schema(self, schema_or_path: PluginJsonSchemaForm) -> None: ...
     @overload
@@ -796,7 +807,7 @@ class CameraStorage(Protocol):
 # StorageController
 @runtime_checkable
 class StorageController(Protocol):
-    async def create_camera_storage(
+    def create_camera_storage(
         self,
         instance: Any,
         camera_id: str,
@@ -865,6 +876,7 @@ class PluginAPI(Protocol):
 
     def on(self, event: APIEventType, listener: Callback) -> "PluginAPI": ...
     def once(self, event: APIEventType, listener: Callback) -> "PluginAPI": ...
+    def off(self, event: APIEventType, listener: Callback) -> "PluginAPI": ...
     def remove_listener(
         self, event: APIEventType, listener: Callback
     ) -> "PluginAPI": ...
@@ -893,7 +905,7 @@ class BasePlugin(ABC):
     @abstractmethod
     def __init__(self, logger: PluginLogger, api_instance: PluginAPI) -> None: ...
     @abstractmethod
-    async def on_form_submit(
+    async def onFormSubmit(
         self, action_id: str, payload: Any
     ) -> Dict[Literal["toastMessage"], str]: ...
     @abstractmethod
